function [c b]=binaryNucleus(R,SIZE) 
% binaryNuclei generates a 3D binary matrix with a spherical shape 
% [c b] = binaryNuclei(R,SIZE) draws a spherical shape with a R radius in 
% an SIZExSIZExSIZE matrix. It returns the distance transform matrix b and
% the binary matrix c
% shape generation

close all;

% dimensions
dimx=SIZE;
dimy=SIZE;
dimz=SIZE;


% center point (initial point for the distance transform function)
xc=round(dimx/2);
yc=round(dimy/2);
zc=round(dimz/2);

% for the distance transform function we need the binary matrix with a
% initial position set to 0 and a LUT (look up table) with the values of 
% the deformation

% creating the matrix
a = ones(dimx,dimy,dimz);
% set the center (initial) point to 0
a(xc,yc,zc)=0;
 


% look up table with the values that are going to deform the sphere
l=createLUT();

% smoothing the look up table 
n=15;
A=l(1:n);
B=l(end-n+1:end);

laux=[B(:);A(:);A(:);B(:)];
laux=gaussf(laux,8);
laux=double(laux);

l(1:n)=laux(n+1:n*2);
l(end-n+1:end)=laux(n*2+1:n*3);
% smoothing it again
l=gaussf(l,3);
l=double(l);

% scaling the look up table between [0,1]
lmax=max(l);
l=l ./ lmax;

% distance transform
b=mydt(a,l);
% choosing the spherical shape with the desired radius R
c=b<R;

%  plot3D(c,'pasive')

