% generateNucleus generates a 3D nucleus cell 
close all;
% parameters of the nucleus
SIZE=50;
radius=30;
blobsI=12;
smooth=6;

% random vectors for the texture
x=rand(SIZE,1);
y=rand(SIZE,1);
z=rand(SIZE,1);

% Shape generation of the nuclei
[c b] = binaryNucleus(radius,SIZE);

% Texture generation
% Smoothing the noise function (texture)
s=0;

for i=1:smooth
    x=2^i*x;
    y=2^i*y;
    z=2^i*z;
    % Perlin noise function
    s = s + noise3d(x,y,z,SIZE)/2^i;
  
end

s2=double(gaussf(s,1.2));
% base noise model
m=c .* s2;

% visualization of the base noise model
h=vol3d('cdata',m,'texture','3D');
view(3);
vol3d(h);
caxis([0.4 0.6]);
colormap('gray');

% Adding the chromatin blobs

% auxiliar matrix
black=zeros(SIZE,SIZE,SIZE);

% random selection of the position of the blobs
for i=1:SIZE*2
    u=randi(SIZE);
    v=randi(SIZE);
    w=randi(SIZE);
    black(u,v,w)=c(u,v,w);
end

% shape of the blobs
nhood=binaryMask(blobsI);
se=strel(nhood);
dilated=imdilate(black,se);
dgauss=double(gaussf(dilated*0.5));

% joining the basic texture and the blobs
mb=(m-(dgauss)).*c;

figure 

% visualization of the final nucleus
h=vol3d('cdata',mb,'texture','3D');
view(3);
vol3d(h);
caxis([0.4 0.6]);
colormap('gray');

% visualization of the nucleus using slides
dip_image(mb)