function img = mydt(input,l)
% mydt is a modified distance transform function.
% img = mydt(input,l) receives an input binary matrix with one pixel 
% (initial point) set to 0 and the rest to 1 and a look-up table l with the 
% values of the deformation we want to apply to the spheres generated with 
% the distance transform function. It returns a new image img with the
% the distance euclidean distance to the initial point.

dimx = length(input(1,:,:));
dimy = length(input(:,1,:));
dimz = length(input(:,:,1));

% the initial point is the center point
x=round(dimx/2);
y=round(dimy/2);
z=round(dimz/2);

img=[dimx dimy dimz];

for i=1:dimx
    for j=1:dimy
        for k=1:dimz
            % Euclidean distance
            r=sqrt((x-i)^2 + (y-j)^2 + 2*(z-k)^2);
            % obtaining the angle between the center and the actual pixel
            alfa=calcAlfa(x,y,i,j);
            
            if alfa == 0
                alfa = 360;
            end
            
            noise=l(alfa);
            % applying the distorsion
            rn=r*(1+noise);
            
            img(i,j,k)=rn;
            
        end
    end
    
end
