function v = noise3d(x,y,z,dim)
% noise3d is a 3D Perlin noise generator.
% v = noise3d(x,y,z,dim) calculates the noise v in the point x,y,z.
% K. Perlin


SIZE = 256;


v=[dim dim dim];

% Gradient vectors

G(1,:)=[1 1 0];
G(2,:)=[-1 1 0];
G(3,:)=[1 -1 0];
G(4,:)=[-1 -1 0];
G(5,:)=[1 0 1];
G(6,:)=[-1 0 1];
G(7,:)=[1 0 -1];
G(8,:)=[-1 0 -1];
G(9,:)=[0 1 1];
G(10,:)=[0 -1 1];
G(11,:)=[0 1 -1];
G(12,:)=[0 -1 -1];
G(13,:)=[1 1 0];
G(14,:)=[-1 1 0];
G(15,:)=[0 -1 1];
G(16,:)=[0 -1 -1];

A=[1:SIZE]';

for i = 1:dim
    
    qx0=floor(x(i));
    qx1=qx0+1;
   
    tx0 = x(i) - floor(x(i));
    tx1 = tx0 - 1;
    
    for j = 1:dim
        
        qy0=floor(y(j));
        qy1=qy0+1;
        
        ty0 = y(j) - floor(y(j));
        ty1 = ty0 - 1;
        
        for k = 1:dim
            %permutation table
            
            p=randperm(SIZE)';
    
                      
            qz0=floor(z(k));
            qz1=qz0+1;
            
            q000= p(mod(qz0 + p(mod(qy0 + p(mod(qx0,SIZE)+1),SIZE)+1),SIZE)+1);
            q001= p(mod(qz1 + p(mod(qy0 + p(mod(qx0,SIZE)+1),SIZE)+1),SIZE)+1);
            q010= p(mod(qz0 + p(mod(qy1 + p(mod(qx0,SIZE)+1),SIZE)+1),SIZE)+1);
            q011= p(mod(qz1 + p(mod(qy1 + p(mod(qx0,SIZE)+1),SIZE)+1),SIZE)+1);
            q100= p(mod(qz0 + p(mod(qy0 + p(mod(qx1,SIZE)+1),SIZE)+1),SIZE)+1);
            q101= p(mod(qz1 + p(mod(qy0 + p(mod(qx1,SIZE)+1),SIZE)+1),SIZE)+1);
            q110= p(mod(qz0 + p(mod(qy1 + p(mod(qx1,SIZE)+1),SIZE)+1),SIZE)+1);
            q111= p(mod(qz1 + p(mod(qy1 + p(mod(qx1,SIZE)+1),SIZE)+1),SIZE)+1);
                 
            
            
            tz0 = z(k) - floor(z(k));
            tz1 = tz0 - 1;
            
            v000 = G(mod(q000,16)+1,1)*tx0 + G(mod(q000,16)+1,2)*ty0 + G(mod(q000,16)+1,3)*tz0;
            v001 = G(mod(q001,16)+1,1)*tx0 + G(mod(q001,16)+1,2)*ty0 + G(mod(q001,16)+1,3)*tz1;
            v010 = G(mod(q010,16)+1,1)*tx0 + G(mod(q010,16)+1,2)*ty1 + G(mod(q010,16)+1,3)*tz0;
            v011 = G(mod(q011,16)+1,1)*tx0 + G(mod(q011,16)+1,2)*ty1 + G(mod(q011,16)+1,3)*tz1;
            v100 = G(mod(q100,16)+1,1)*tx1 + G(mod(q100,16)+1,2)*ty0 + G(mod(q100,16)+1,3)*tz0;
            v101 = G(mod(q101,16)+1,1)*tx1 + G(mod(q101,16)+1,2)*ty0 + G(mod(q101,16)+1,3)*tz1;
            v110 = G(mod(q110,16)+1,1)*tx1 + G(mod(q110,16)+1,2)*ty1 + G(mod(q110,16)+1,3)*tz0;
            v111 = G(mod(q111,16)+1,1)*tx1 + G(mod(q111,16)+1,2)*ty1 + G(mod(q111,16)+1,3)*tz1;
            
            
            sx = (10 - 15*tx0 + 6 * tx0*tx0)*tx0*tx0*tx0;
            
            v00=v000 - sx*(v000-v100);
            v01=v001 - sx*(v001-v101);
            v10=v011 - sx*(v011-v111);
            v11=v010 - sx*(v010-v110);
            
            sy = (10 - 15*ty0 + 6 * ty0*ty0)*ty0*ty0*ty0;
            
            v0=v00 - sy*(v00-v11);
            v1=v01 - sy*(v01-v10);
            
            sz = (10 - 15*tz0 + 6 *tz0* tz0)*tz0*tz0*tz0;
            
            v(i,j,k)= v0 - sz*(v0-v1);
            
                      
        end
    end
end

v = (v+1)/2;





